from .util import AttrCheck
import storm.locals as rain


class TException(rain.Storm, AttrCheck):
    __storm_table__ = "Exceptions"
    exception_id = rain.Int("ExceptionId", primary=True)
    name = rain.Unicode("Name")
    fail = rain.Bool("Fail")


class TExecution(rain.Storm, AttrCheck):
    __storm_table__ = "Executions"
    execution_id = rain.Int("ExecutionId", primary=True)
    start_date = rain.DateTime("StartDate")
    finish_date = rain.DateTime("FinishDate")
    completed = rain.Bool("Completed")


class TAddress(rain.Storm, AttrCheck):
    __storm_table__ = "Ips"
    address_id = rain.Int("IpId", primary=True)
    str = rain.Unicode("AsStr")
    ip = rain.Int("AsInt")
    china = rain.Bool("China")
    block_target = rain.Bool("BlockTarget")
    server = rain.Bool("Server")
    redirect_target = rain.Bool("RedirectTarget")
    gfw_target = rain.Bool("GfwTarget")


class TKind(rain.Storm, AttrCheck):
    __storm_table__ = "Kinds"
    kind_id = rain.Int("KindId", primary=True)
    name = rain.Unicode("Name")
    xxid = rain.Int("ExpectExceptionId")
    exception = rain.Reference(xxid, "TException.exception_id")


class TDomain(rain.Storm, AttrCheck):
    __storm_table__ = "Domains"
    domain_id = rain.Int("DomainId", primary=True)
    name = rain.Unicode("Name")
    kind_id = rain.Int("KindId")
    kind = rain.Reference(kind_id, "TKind.kind_id")


class TRecord(rain.Storm, AttrCheck):
    __storm_table__ = "Records"
    row_id = rain.Int("rowid", primary=True)
    record_id = rain.Int("RecordId")
    address_id = rain.Int("IpId")
    address = rain.Reference(address_id, "TAddress.address_id")


class TMessage(rain.Storm, AttrCheck):
    __storm_table__ = "Messages"
    message_id = rain.Int("MessageId", primary=True)
    execution_id = rain.Int("ExecutionId")
    server_id = rain.Int("ServerId")
    domain_id = rain.Int("DomainId")
    record_id = rain.Int("RecordId")
    exception_id = rain.Int("ExceptionId")
    failed = rain.Bool("Failed")
    execution = rain.Reference(execution_id, "TExecution.execution_id")
    server = rain.Reference(server_id, "TAddress.address_id")
    domain = rain.Reference(domain_id, "TDomain.domain_id")
    exception = rain.Reference(exception_id, "TException.exception_id")
